@@ -20,8 +20,11 @@
  *	Sebastien Pouliot  <sebastien@ximian.com>
  */
 
+#include "gdiplus-private.h"
 #include "icocodec.h"
 
+GUID gdip_ico_image_format_guid = {0xb96b3cb5U, 0x0728U, 0x11d3U, {0x9d, 0x7b, 0x00, 0x00, 0xf8, 0x1e, 0xf3, 0x2e}};
+
 /* Codecinfo related data*/
 static ImageCodecInfo ico_codec;
 static const WCHAR ico_codecname[] = {'B', 'u', 'i','l', 't', '-','i', 'n', ' ', 'I', 'C', 'O', 0}; /* Built-in ICO */
@@ -42,7 +45,7 @@ gdip_getcodecinfo_ico ()
 	ico_codec.FormatDescription = (const WCHAR*) ico_format;
 	ico_codec.FilenameExtension = (const WCHAR*) ico_extension;
 	ico_codec.MimeType = (const WCHAR*) ico_mimetype;
-	ico_codec.Flags = Decoder | SupportBitmap | Builtin;
+	ico_codec.Flags = ImageCodecFlagsDecoder | ImageCodecFlagsSupportBitmap | ImageCodecFlagsBuiltin;
 	ico_codec.Version = 1;
 	ico_codec.SigCount = 1;
 	ico_codec.SigSize = 4;
@@ -89,9 +92,9 @@ read_ICONDIRENTRY (void *pointer, ICONDIRENTRY *entry, ImageSource source, BOOL
 		b = (BYTE*)&entry->wBitCount;
 		entry->wBitCount = (b[1] << 8) | b[0];
 		b = (BYTE*)&entry->dwBytesInRes;
-		entry->dwBytesInRes = (b[3] << 24) | (b[2] << 16) | (b[1] << 8) | b[0];
+		entry->dwBytesInRes = ((guint32)b[3] << 24) | (b[2] << 16) | (b[1] << 8) | b[0];
 		b = (BYTE*)&entry->dwImageOffset;
-		entry->dwImageOffset = (b[3] << 24) | (b[2] << 16) | (b[1] << 8) | b[0];
+		entry->dwImageOffset = ((guint32)b[3] << 24) | (b[2] << 16) | (b[1] << 8) | b[0];
 	}
 #endif
 	return TRUE;
@@ -102,7 +105,7 @@ gdip_read_ico_image_from_file_stream (void *pointer, GpImage **image, ImageSourc
 {
 	GpStatus status = InvalidParameter;
 	GpBitmap *result = NULL;
-	guchar *pixels = NULL;
+	BYTE *pixels = NULL;
 	WORD w, count;
 	void *p = &w;
 	BYTE *b = (BYTE*)&w;
@@ -165,15 +168,14 @@ gdip_read_ico_image_from_file_stream (void *pointer, GpImage **image, ImageSourc
 		goto error;
 	
 	result = gdip_bitmap_new_with_frame (NULL, TRUE);
-	result->type = imageBitmap;
+	result->type = ImageTypeBitmap;
 	result->image_format = ICON;
-	result->active_bitmap->pixel_format = Format32bppArgb; /* icons are always promoted to 32 bbp */
+	result->active_bitmap->pixel_format = PixelFormat32bppARGB; /* icons are always promoted to 32 bbp */
 	result->active_bitmap->width = entry.bWidth;
 	result->active_bitmap->height = entry.bHeight;
 	result->active_bitmap->stride = result->active_bitmap->width * 4;
-	/* Ensure pixman_bits_t alignment */
-	result->active_bitmap->stride += (sizeof(pixman_bits_t) - 1);
-	result->active_bitmap->stride &= ~(sizeof(pixman_bits_t) - 1);
+	/* Ensure 32bits alignment */
+	gdip_align_stride (result->active_bitmap->stride);
 	result->active_bitmap->dpi_horz = 96.0f;
 	result->active_bitmap->dpi_vert = 96.0f;
 
@@ -289,7 +291,7 @@ gdip_read_ico_image_from_file_stream (void *pointer, GpImage **image, ImageSourc
 			} else {
 				BYTE *line_data = xor_data + y * line_xor_length + x * 4;
 				/* ARGB to BRGA */
-				color = (line_data [0] | line_data [1] << 8 | line_data [2] << 16 | line_data [3] << 24);
+				color = (line_data [0] | line_data [1] << 8 | line_data [2] << 16 | (guint32)line_data [3] << 24);
 			}
 			/* image is reversed (y) */
 			GdipBitmapSetPixel (result, x, entry.bHeight - y - 1, color);
