@@ -221,6 +221,9 @@ gdip_add_rect_to_array_notcontained (GpRectF** srcarray, int* elements,  GpRectF
 static BOOL
 gdip_equals (GpRectF *rect1, GpRectF *rect2)
 {
+	if (!rect1)
+		return (rect2 == NULL);
+
 	return (rect1->X == rect2->X &&
 		rect1->Width == rect2->Width &&
 		rect1->Y == rect2->Y &&
@@ -404,7 +407,7 @@ gdip_createRegion (GpRegion **region, RegionType type, void *src)
 */
 
 // coverity[+alloc : arg-*0]
-GpStatus
+GpStatus WINGDIPAPI
 GdipCreateRegion (GpRegion **region)
 {
         if (!region)
@@ -414,7 +417,7 @@ GdipCreateRegion (GpRegion **region)
 }
 
 // coverity[+alloc : arg-*1]
-GpStatus
+GpStatus WINGDIPAPI
 GdipCreateRegionRect (GDIPCONST GpRectF *rect, GpRegion **region)
 {
         if (!region || !rect)
@@ -424,7 +427,7 @@ GdipCreateRegionRect (GDIPCONST GpRectF *rect, GpRegion **region)
 }
 
 // coverity[+alloc : arg-*1]
-GpStatus
+GpStatus WINGDIPAPI
 GdipCreateRegionRectI (GDIPCONST GpRect *rect, GpRegion **region)
 {
         if (!region || !rect)
@@ -434,8 +437,8 @@ GdipCreateRegionRectI (GDIPCONST GpRect *rect, GpRegion **region)
 }
 
 // coverity[+alloc : arg-*2]
-GpStatus
-GdipCreateRegionRgnData (GDIPCONST BYTE *regionData, int size, GpRegion **region)
+GpStatus WINGDIPAPI
+GdipCreateRegionRgnData (GDIPCONST BYTE *regionData, INT size, GpRegion **region)
 {
 	GpRegion *result;
 
@@ -490,7 +493,7 @@ GdipCreateRegionRgnData (GDIPCONST BYTE *regionData, int size, GpRegion **region
 }
 
 // coverity[+alloc : arg-*1]
-GpStatus
+GpStatus WINGDIPAPI
 GdipCloneRegion (GpRegion *region, GpRegion **cloneRegion)
 {
         GpRegion *result;
@@ -508,7 +511,7 @@ GdipCloneRegion (GpRegion *region, GpRegion **cloneRegion)
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipDeleteRegion (GpRegion *region)
 {
         if (!region)
@@ -521,7 +524,7 @@ GdipDeleteRegion (GpRegion *region)
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetInfinite (GpRegion *region)
 {
         GpRectF rect;
@@ -540,7 +543,7 @@ GdipSetInfinite (GpRegion *region)
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipSetEmpty (GpRegion *region)
 {
         if (!region)
@@ -552,6 +555,27 @@ GdipSetEmpty (GpRegion *region)
         return Ok;
 }
 
+/* pre-process negative width and height, without modifying the originals, see bug #383878 */
+static void
+gdip_normalize_rectangle (GpRectF *rect, GpRectF *normalized)
+{
+	if (rect->Width < 0) {
+		normalized->X = rect->X + rect->Width;
+		normalized->Width = fabs (rect->Width);
+	} else {
+		normalized->X = rect->X;
+		normalized->Width = rect->Width;
+	}
+
+	if (rect->Height < 0) {
+		normalized->Y = rect->Y + rect->Height;
+		normalized->Height = fabs (rect->Height);
+	} else {
+		normalized->Y = rect->Y;
+		normalized->Height = rect->Height;
+	}
+}
+
 /* Exclude */
 static void
 gdip_combine_exclude (GpRegion *region, GpRectF *rtrg, int cntt)
@@ -567,8 +591,12 @@ gdip_combine_exclude (GpRegion *region, GpRectF *rtrg, int cntt)
                 gdip_add_rect_to_array (&allsrcrects, &allsrccnt, rect);
 
 	/* Create the list of target rectangles to process, it will contain splitted ones later */
-        for (i = 0, rect = rtrg; i < cntt; i++, rect++)
-                gdip_add_rect_to_array (&alltrgrects, &alltrgcnt, rect);
+        for (i = 0, rect = rtrg; i < cntt; i++, rect++) {
+		/* normalize */
+		GpRectF normal;
+		gdip_normalize_rectangle (rect, &normal);
+		gdip_add_rect_to_array (&alltrgrects, &alltrgcnt, &normal);
+	}
 
 	/* Init current with the first element in the array */
 	current.X = REGION_INFINITE_POSITION - 1;
@@ -602,6 +630,8 @@ gdip_combine_exclude (GpRegion *region, GpRectF *rtrg, int cntt)
 			newrect.Y = current.Y;
 			if (current.Y >= recttrg->Y) {  /* Our rect intersects in the upper part with another rect */
 				newrect.Height = MIN (recttrg->Y + recttrg->Height - current.Y, current.Height);
+				if (newrect.Height < 0)
+					newrect.Height = current.Height;
 
 				if (current.X >= recttrg->X) { /* Hit from behind */
 					newrect.X = recttrg->X + recttrg->Width;
@@ -683,8 +713,12 @@ gdip_combine_complement (GpRegion *region, GpRectF *rtrg, int cntt)
         int allsrccnt = 0, i,  trgcnt;
 
 	/* Create the list of source rectangles to process */
-        for (i = 0, rect = rtrg; i < cntt; i++, rect++)
-                gdip_add_rect_to_array (&allsrcrects, &allsrccnt, rect);
+        for (i = 0, rect = rtrg; i < cntt; i++, rect++) {
+		/* normalize */
+		GpRectF normal;
+		gdip_normalize_rectangle (rect, &normal);
+		gdip_add_rect_to_array (&allsrcrects, &allsrccnt, &normal);
+	}
 
 	regsrc.rects = allsrcrects;
 	regsrc.cnt = allsrccnt;
@@ -717,8 +751,12 @@ gdip_combine_union (GpRegion *region, GpRectF *rtrg, int cnttrg)
         for (i = 0, rect = region->rects; i < region->cnt; i++, rect++)
                 gdip_add_rect_to_array (&allrects, &allcnt,  rect);
 
-        for (i = 0, rect = rtrg; i < cnttrg; i++, rect++)
-                gdip_add_rect_to_array (&allrects, &allcnt,  rect);
+        for (i = 0, rect = rtrg; i < cnttrg; i++, rect++) {
+		/* normalize */
+		GpRectF normal;
+		gdip_normalize_rectangle (rect, &normal);
+		gdip_add_rect_to_array (&allrects, &allcnt, &normal);
+	}
 
         if (allcnt == 0) {
                 GdipFree (allrects);
@@ -861,21 +899,23 @@ gdip_combine_intersect (GpRegion *region, GpRectF *rtrg, int cnttrg)
 
 	for (rectsrc = region->rects, src = 0; src < region->cnt; src++, rectsrc++) {
 		for (recttrg = rtrg, trg = 0; trg < cnttrg; trg++, recttrg++) {
+			/* normalize */
+			GpRectF normal;
+			gdip_normalize_rectangle (recttrg, &normal);
+
 			/* Intersects With */
-			if (rectsrc->X >= recttrg->X + recttrg->Width ||
-				rectsrc->X + rectsrc->Width <= recttrg->X ||
-				rectsrc->Y >= recttrg->Y + recttrg->Height ||
-				rectsrc->Y + rectsrc->Height <= recttrg->Y) { /* BUG, TODO: Re-test*/
+			if ((rectsrc->X >= normal.X + normal.Width) || (rectsrc->X + rectsrc->Width <= normal.X) ||
+				(rectsrc->Y >= normal.Y + normal.Height) || (rectsrc->Y + rectsrc->Height <= normal.Y)) {
 				continue;
 			}
 			/* Area that intersects */
-			rectcur.X = rectsrc->X > recttrg->X ? rectsrc->X : recttrg->X;
-			rectcur.Y = rectsrc->Y > recttrg->Y ? rectsrc->Y : recttrg->Y;
-			rectcur.Width = rectsrc->X + rectsrc->Width < recttrg->X + recttrg->Width ?
-				rectsrc->X + rectsrc->Width - rectcur.X : recttrg->X + recttrg->Width - rectcur.X;
+			rectcur.X = rectsrc->X > normal.X ? rectsrc->X : normal.X;
+			rectcur.Y = rectsrc->Y > normal.Y ? rectsrc->Y : normal.Y;
+			rectcur.Width = rectsrc->X + rectsrc->Width < normal.X + normal.Width ?
+				rectsrc->X + rectsrc->Width - rectcur.X : normal.X + normal.Width - rectcur.X;
 
-			rectcur.Height = rectsrc->Y + rectsrc->Height < recttrg->Y + recttrg->Height ?
-				rectsrc->Y + rectsrc->Height - rectcur.Y : recttrg->Y + recttrg->Height - rectcur.Y;
+			rectcur.Height = rectsrc->Y + rectsrc->Height < normal.Y + normal.Height ?
+				rectsrc->Y + rectsrc->Height - rectcur.Y : normal.Y + normal.Height - rectcur.Y;
 
 			/* Combine with previous areas that intersect with rect */
 			gdip_combine_union (&regunion, &rectcur, 1);
@@ -902,8 +942,12 @@ gdip_combine_xor (GpRegion *region, GpRectF *recttrg, int cnttrg)
         for (i = 0, rect = region->rects; i < region->cnt; i++, rect++)
                 gdip_add_rect_to_array (&allrects, &allcnt,  rect);
 
-        for (i = 0, rect = recttrg; i < cnttrg; i++, rect++)
-                gdip_add_rect_to_array (&allrects, &allcnt,  rect);
+	for (i = 0, rect = recttrg; i < cnttrg; i++, rect++) {
+		/* normalize */
+		GpRectF normal;
+		gdip_normalize_rectangle (rect, &normal);
+		gdip_add_rect_to_array (&allrects, &allcnt, &normal);
+	}
 
 	rgnsrc = (GpRegion *) GdipAlloc (sizeof (GpRegion));
 	rgnsrc->type = RegionTypeRectF;
@@ -927,7 +971,7 @@ gdip_combine_xor (GpRegion *region, GpRectF *recttrg, int cnttrg)
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipCombineRegionRect (GpRegion *region, GDIPCONST GpRectF *rect, CombineMode combineMode)
 {
         if (!region || !rect)
@@ -986,7 +1030,7 @@ GdipCombineRegionRect (GpRegion *region, GDIPCONST GpRectF *rect, CombineMode co
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipCombineRegionRectI (GpRegion *region, GDIPCONST GpRect *recti, CombineMode combineMode)
 {
         GpRectF rect;
@@ -1037,7 +1081,7 @@ gdip_combine_exclude_from_infinite (GpRegion *region, GpPath *path)
 	return TRUE;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipCombineRegionPath (GpRegion *region, GpPath *path, CombineMode combineMode)
 {
 	GpRegionBitmap *path_bitmap, *result;
@@ -1186,7 +1230,7 @@ gdip_combine_pathbased_region (GpRegion *region1, GpRegion *region2, CombineMode
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipCombineRegionRegion (GpRegion *region,  GpRegion *region2, CombineMode combineMode)
 {
         if (!region || !region2)
@@ -1277,7 +1321,7 @@ GdipCombineRegionRegion (GpRegion *region,  GpRegion *region2, CombineMode combi
         return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetRegionBounds (GpRegion *region, GpGraphics *graphics, GpRectF *rect)
 {
         if (!region || !graphics || !rect)
@@ -1306,7 +1350,7 @@ GdipGetRegionBounds (GpRegion *region, GpGraphics *graphics, GpRectF *rect)
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipIsEmptyRegion (GpRegion *region, GpGraphics *graphics, BOOL *result)
 {
         if (!region || !graphics || !result)
@@ -1317,7 +1361,7 @@ GdipIsEmptyRegion (GpRegion *region, GpGraphics *graphics, BOOL *result)
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipIsInfiniteRegion (GpRegion *region, GpGraphics *graphics, BOOL *result)
 {
       if (!region || !graphics || !result)
@@ -1328,7 +1372,7 @@ GdipIsInfiniteRegion (GpRegion *region, GpGraphics *graphics, BOOL *result)
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipIsVisibleRegionPoint (GpRegion *region, float x, float y, GpGraphics *graphics, BOOL *result)
 {
         if (!region || !result)
@@ -1347,14 +1391,14 @@ GdipIsVisibleRegionPoint (GpRegion *region, float x, float y, GpGraphics *graphi
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipIsVisibleRegionPointI (GpRegion *region, int x, int y, GpGraphics *graphics, BOOL *result)
 {
         return GdipIsVisibleRegionPoint (region, x, y, graphics, result);
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipIsVisibleRegionRect (GpRegion *region, float x, float y, float width, float height, GpGraphics *graphics, BOOL *result)
 {
         BOOL found = FALSE;
@@ -1404,14 +1448,14 @@ GdipIsVisibleRegionRect (GpRegion *region, float x, float y, float width, float
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipIsVisibleRegionRectI (GpRegion *region, int x, int y, int width, int height, GpGraphics *graphics, BOOL *result)
 {
         return GdipIsVisibleRegionRect (region, x, y, width, height, graphics, result);
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetRegionScansCount (GpRegion *region, int* count, GpMatrix* matrix)
 {
 	GpRegion *work = NULL;
@@ -1467,7 +1511,7 @@ GdipGetRegionScansCount (GpRegion *region, int* count, GpMatrix* matrix)
 	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetRegionScans (GpRegion *region, GpRectF* rects, int* count, GpMatrix* matrix)
 {
 	GpRegion *work = NULL;
@@ -1524,7 +1568,7 @@ GdipGetRegionScans (GpRegion *region, GpRectF* rects, int* count, GpMatrix* matr
 	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipIsEqualRegion (GpRegion *region, GpRegion *region2, GpGraphics *graphics, BOOL *result)
 {
         int i;
@@ -1576,7 +1620,7 @@ GdipIsEqualRegion (GpRegion *region, GpRegion *region2, GpGraphics *graphics, BO
         return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipTranslateRegion (GpRegion *region, float dx, float dy)
 {
         if (!region)
@@ -1607,7 +1651,7 @@ GdipTranslateRegion (GpRegion *region, float dx, float dy)
         return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipTranslateRegionI (GpRegion *region, int dx, int dy)
 {
         return GdipTranslateRegion (region, dx, dy);
@@ -1634,7 +1678,7 @@ ScaleRegion (GpRegion *region, float sx, float sy)
         return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipTransformRegion (GpRegion *region, GpMatrix *matrix)
 {
 	GpStatus status = Ok;
@@ -1695,7 +1739,7 @@ GdipTransformRegion (GpRegion *region, GpMatrix *matrix)
 }
 
 // coverity[+alloc : arg-*1]
-GpStatus
+GpStatus WINGDIPAPI
 GdipCreateRegionPath (GpPath *path, GpRegion **region)
 {
         if (!region || !path)
@@ -1743,7 +1787,7 @@ GdipCreateRegionPath (GpPath *path, GpRegion **region)
  *		byte[Size2]		branch #2
  */
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetRegionDataSize (GpRegion *region, UINT *bufferSize)
 {
 	if (!region || !bufferSize)
@@ -1765,7 +1809,7 @@ GdipGetRegionDataSize (GpRegion *region, UINT *bufferSize)
 }
 
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetRegionData (GpRegion *region, BYTE *buffer, UINT bufferSize, UINT *sizeFilled)
 {
 	GpStatus status;
@@ -1811,7 +1855,7 @@ GdipGetRegionData (GpRegion *region, BYTE *buffer, UINT bufferSize, UINT *sizeFi
 	return Ok;
 }
 
-GpStatus
+GpStatus WINGDIPAPI
 GdipGetRegionHRgn (GpRegion *region, GpGraphics *graphics, HRGN *hRgn)
 {
 	if (!region || !graphics || !hRgn)
@@ -1829,7 +1873,7 @@ GdipGetRegionHRgn (GpRegion *region, GpGraphics *graphics, HRGN *hRgn)
 }
 
 // coverity[+alloc : arg-*1]
-GpStatus
+GpStatus WINGDIPAPI
 GdipCreateRegionHrgn (HRGN hRgn, GpRegion **region)
 {
 	if (!hRgn || !region)
